@@ -4,10 +4,12 @@
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published
- * by the Free Software Foundation; either version 2 of the licence or (at
+ * by the Free Software Foundation; either version 2.1 of the licence or (at
  * your option) any later version.
  *
  * See the included COPYING file for more information.
+ *
+ * Author: Stef Walter <stefw@gnome.org>
  */
 
 #include "config.h"
@@ -24,6 +26,7 @@ static const SecretSchema network_schema = {
 		{  "protocol", SECRET_SCHEMA_ATTRIBUTE_STRING },
 		{  "port", SECRET_SCHEMA_ATTRIBUTE_INTEGER },
 		{  "server", SECRET_SCHEMA_ATTRIBUTE_STRING },
+		{  "authtype", SECRET_SCHEMA_ATTRIBUTE_STRING },
 		{  "NULL", 0 },
 	}
 };
